using System;
using System.Collections.ObjectModel;
using Microsoft.Interop.Security.AzRoles;
using System.Security.Principal;

namespace Microsoft.AzRoles
{
    public class Application
    {
        IAzApplication application;

        internal Application(IAzApplication application)
        {
            this.application = application;
        }

        public void AddDelegatedPolicyUser(string delegatedPolicyUser)
        {
            application.AddDelegatedPolicyUser(delegatedPolicyUser, null);
        }

        public void AddDelegatedPolicyUserName(string delegatedPolicyUser)
        {
            application.AddDelegatedPolicyUserName(delegatedPolicyUser, null);
        }

        public void AddPolicyAdministrator(string admin)
        {
            application.AddPolicyAdministrator(admin, null);
        }

        public void AddPolicyAdministratorName(string admin)
        {
            application.AddPolicyAdministratorName(admin, null);
        }

        public void AddPolicyReader(string reader)
        {
            application.AddPolicyReader(reader, null);
        }

        public void AddPolicyReaderName(string reader)
        {
            application.AddPolicyReaderName(reader, null);
        }

        public void AddPropertyItem(int propertyId, object propertyValue)
        {
            application.AddPropertyItem(propertyId, propertyValue, null);
        }

        public string ApplicationData
        {
            get { return application.ApplicationData; }
            set { application.ApplicationData = value; }
        }

        public ReadOnlyCollection<ApplicationGroup> ApplicationGroups
        {
            get { return Utils.ToApplicationGroupArray(application.ApplicationGroups); }
        }

        public bool ApplyStoreSacl
        {
            get { return Convert.ToBoolean(application.ApplyStoreSacl); }
            set { application.ApplyStoreSacl = Convert.ToInt32(value); }
        }

        public string AuthzInterfaceClsid
        {
            get { return application.AuthzInterfaceClsid; }
            set { application.AuthzInterfaceClsid = value; }
        }

        public ApplicationGroup CreateApplicationGroup(string groupName)
        {
            return new ApplicationGroup(application.CreateApplicationGroup(groupName, null));
        }

        public Operation CreateOperation(string operationName)
        {
            return new Operation(application.CreateOperation(operationName, null));
        }

        public Role CreateRole(string roleName)
        {
            return new Role(application.CreateRole(roleName, null));
        }

        public Scope CreateScope(string scopeName)
        {
            return new Scope(application.CreateScope(scopeName, null));
        }

        public Task CreateTask(string taskName)
        {
            return new Task(application.CreateTask(taskName, null));
        }

        public ReadOnlyCollection<string> DelegatedPolicyUsers
        {
            get { return Utils.ToStringArray(application.DelegatedPolicyUsers); }
        }

        public ReadOnlyCollection<string> DelegatedPolicyUsersName
        {
            get { return Utils.ToStringArray(application.DelegatedPolicyUsersName); }
        }

        public void DeleteApplicationGroup(string groupName)
        {
            application.DeleteApplicationGroup(groupName, null);
        }

        public void DeleteDelegatedPolicyUser(string delegatedPolicyUser)
        {
            application.DeleteDelegatedPolicyUser(delegatedPolicyUser, null);
        }

        public void DeleteDelegatedPolicyUserName(string delegatedPolicyUser)
        {
            application.DeleteDelegatedPolicyUserName(delegatedPolicyUser, null);
        }

        public void DeleteOperation(string operationName)
        {
            application.DeleteOperation(operationName, null);
        }

        public void DeletePolicyAdministrator(string admin)
        {
            application.DeletePolicyAdministrator(admin, null);
        }

        public void DeletePolicyAdministratorName(string admin)
        {
            application.DeletePolicyAdministratorName(admin, null);
        }

        public void DeletePolicyReader(string reader)
        {
            application.DeletePolicyReader(reader, null);
        }

        public void DeletePolicyReaderName(string reader)
        {
            application.DeletePolicyReaderName(reader, null);
        }

        public void DeletePropertyItem(int propertyId, object propertyValue)
        {
            application.DeletePropertyItem(propertyId, propertyValue, null);
        }

        public void DeleteRole(string roleName)
        {
            application.DeleteRole(roleName, null);
        }

        public void DeleteScope(string scopeName)
        {
            application.DeleteScope(scopeName, null);
        }

        public void DeleteTask(string taskName)
        {
            application.DeleteTask(taskName, null);
        }

        public bool GenerateAudits
        {
            get { return Convert.ToBoolean(application.GenerateAudits); }
            set { application.GenerateAudits = Convert.ToInt32(value); }
        }

        public object GetProperty(int propertyId)
        {
            return application.GetProperty(propertyId, null);
        }

        public ClientContext InitializeClientContext()
        {
            return new ClientContext(application.InitializeClientContextFromToken((ulong)WindowsIdentity.GetCurrent().Token.ToInt64(), null));
        }

        public ClientContext InitializeClientContext(string clientName, string domainName)
        {
            return new ClientContext(application.InitializeClientContextFromName(clientName, domainName, null));
        }

        public ClientContext InitializeClientContext(SecurityIdentifier sid)
        {
            return new ClientContext(application.InitializeClientContextFromStringSid(sid.Value, 0, null));
        }

        public ClientContext InitializeClientContext(WindowsIdentity windowsIdentity)
        {
            return new ClientContext(application.InitializeClientContextFromToken((ulong)windowsIdentity.Token.ToInt64(), null));
        }

        public ClientContext InitializeClientContext(IntPtr token)
        {
            return new ClientContext(application.InitializeClientContextFromToken((ulong)token.ToInt64(), null));
        }

        public string Name
        {
            get { return application.Name; }
            set { application.Name = value; }
        }

        public ApplicationGroup OpenApplicationGroup(string groupName)
        {
            return new ApplicationGroup(application.OpenApplicationGroup(groupName, null));
        }

        public Operation OpenOperation(string operationName)
        {
            return new Operation(application.OpenOperation(operationName, null));
        }

        public Role OpenRole(string roleName)
        {
            return new Role(application.OpenRole(roleName, null));
        }

        public Scope OpenScope(string scopeName)
        {
            return new Scope(application.OpenScope(scopeName, null));
        }

        public Task OpenTask(string taskName)
        {
            return new Task(application.OpenTask(taskName, null));
        }

        public ReadOnlyCollection<Operation> Operations
        {
            get { return Utils.ToOperationArray(application.Operations); }
        }

        public ReadOnlyCollection<string> PolicyAdministrators
        {
            get { return Utils.ToStringArray(application.PolicyAdministrators); }
        }

        public ReadOnlyCollection<string> PolicyAdministratorsName
        {
            get { return Utils.ToStringArray(application.PolicyAdministratorsName); }
        }

        public ReadOnlyCollection<string> PolicyReaders
        {
            get { return Utils.ToStringArray(application.PolicyReaders); }
        }

        public ReadOnlyCollection<string> PolicyReadersName
        {
            get { return Utils.ToStringArray(application.PolicyReadersName); }
        }

        public ReadOnlyCollection<Role> Roles
        {
            get { return Utils.ToRoleArray(application.Roles); }
        }

        public ReadOnlyCollection<Scope> Scopes
        {
            get { return Utils.ToScopeArray(application.Scopes); }
        }

        public void SetProperty(int propertyId, object propertyValue)
        {
            application.SetProperty(propertyId, propertyValue, null);
        }

        public void Submit(int flags)
        {
            application.Submit(flags, null);
        }

        public ReadOnlyCollection<Task> Tasks
        {
            get { return Utils.ToTaskArray(application.Tasks); }
        }

        public string Version
        {
            get { return application.Version; }
            set { application.Version = value; }
        }

        public bool Writable
        {
            get { return Convert.ToBoolean(application.Writable); }
        }

        public override string ToString()
        {
            return Name;
        }
    }
}
