using System;
using System.Collections.ObjectModel;
using Microsoft.Interop.Security.AzRoles;

namespace Microsoft.AzRoles
{
    public class AuthorizationStore
    {
        AzAuthorizationStoreClass store;

        public AuthorizationStore(string policyUrl)
            : this(policyUrl, 0)
        {
        }

        public AuthorizationStore(string policyUrl, int flags)
        {
            store = new AzAuthorizationStoreClass();
            store.Initialize(flags, policyUrl, null);
        }

        public void AddDelegatedPolicyUser(string delegatedPolicyUser)
        {
            store.AddDelegatedPolicyUser(delegatedPolicyUser, null);
        }

        public void AddDelegatedPolicyUserName(string delegatedPolicyUserName)
        {
            store.AddDelegatedPolicyUserName(delegatedPolicyUserName, null);
        }

        public void AddPolicyAdministrator(string admin)
        {
            store.AddPolicyAdministrator(admin, null);
        }

        public void AddPolicyAdministratorName(string adminName)
        {
            store.AddPolicyAdministratorName(adminName, null);
        }

        public void AddPolicyReader(string reader)
        {
            store.AddPolicyReader(reader, null);
        }

        public void AddPolicyReaderName(string readerName)
        {
            store.AddPolicyReaderName(readerName, null);
        }

        public void AddPropertyItem(int propertyId, object propertyValue)
        {
            store.AddPropertyItem(propertyId, propertyValue, null);
        }

        public string ApplicationData
        {
            get { return store.ApplicationData; }
            set { store.ApplicationData = value; }
        }

        public ReadOnlyCollection<ApplicationGroup> ApplicationGroups
        {
            get { return Utils.ToApplicationGroupArray(store.ApplicationGroups); }
        }

        public ReadOnlyCollection<Application> Applications
        {
            get { return Utils.ToApplicationArray(store.Applications); }
        }

        public bool ApplyStoreSacl
        {
            get { return Convert.ToBoolean(store.ApplyStoreSacl); }
            set { store.ApplyStoreSacl = Convert.ToInt32(value); }
        }

        public Application CreateApplication(string applicationName)
        {
            return new Application(store.CreateApplication(applicationName, null));
        }

        public ApplicationGroup CreateApplicationGroup(string groupName)
        {
            return new ApplicationGroup(store.CreateApplicationGroup(groupName, null));
        }

        public ReadOnlyCollection<string> DelegatedPolicyUsers
        {
            get { return Utils.ToStringArray(store.DelegatedPolicyUsers); }
        }

        public ReadOnlyCollection<string> DelegatedPolicyUsersName
        {
            get { return Utils.ToStringArray(store.DelegatedPolicyUsersName); }
        }

        public void Delete()
        {
            store.Delete(null);
        }

        public void DeleteApplication(string applicationName)
        {
            store.DeleteApplication(applicationName, null);
        }

        public void DeleteApplicationGroup(string groupName)
        {
            store.DeleteApplicationGroup(groupName, null);
        }

        public void DeleteDelegatedPolicyUser(string delegatedPolicyUser)
        {
            store.DeleteDelegatedPolicyUser(delegatedPolicyUser, null);
        }

        public void DeleteDelegatedPolicyUserName(string delegatedPolicyUserName)
        {
            store.DeleteDelegatedPolicyUserName(delegatedPolicyUserName, null);
        }

        public void DeletePolicyAdministrator(string admin)
        {
            store.DeletePolicyAdministrator(admin, null);
        }

        public void DeletePolicyAdministratorName(string adminName)
        {
            store.DeletePolicyAdministratorName(adminName, null);
        }

        public void DeletePolicyReader(string reader)
        {
            store.DeletePolicyReader(reader, null);
        }

        public void DeletePolicyReaderName(string readerName)
        {
            store.DeletePolicyReaderName(readerName, null);
        }

        public void DeletePropertyItem(int propertyId, object propertyValue)
        {
            store.DeletePropertyItem(propertyId, propertyValue, null);
        }

        public string Description
        {
            get { return store.Description; }
            set { store.Description = value; }
        }

        public int DomainTimeout
        {
            get { return store.DomainTimeout; }
            set { store.DomainTimeout = value; }
        }

        public bool GenerateAudits
        {
            get { return Convert.ToBoolean(store.GenerateAudits); }
            set { store.GenerateAudits = Convert.ToInt32(value); }
        }

        public object GetProperty(int propertyId)
        {
            return store.GetProperty(propertyId, null);
        }

        public int MaxScriptEngines
        {
            get { return store.MaxScriptEngines; }
            set { store.MaxScriptEngines = value; }
        }

        public Application OpenApplication(string applicationName)
        {
            return new Application(store.OpenApplication(applicationName, null));
        }

        public ApplicationGroup OpenApplicationGroup(string groupName)
        {
            return new ApplicationGroup(store.OpenApplicationGroup(groupName, null));
        }

        public ReadOnlyCollection<string> PolicyAdministrators
        {
            get { return Utils.ToStringArray(store.PolicyAdministrators); }
        }

        public ReadOnlyCollection<string> PolicyAdministratorsName
        {
            get { return Utils.ToStringArray(store.PolicyAdministratorsName); }
        }

        public ReadOnlyCollection<string> PolicyReaders
        {
            get { return Utils.ToStringArray(store.PolicyReaders); }
        }

        public ReadOnlyCollection<string> PolicyReadersName
        {
            get { return Utils.ToStringArray(store.PolicyReadersName); }
        }

        public int ScriptEngineTimeout
        {
            get { return store.ScriptEngineTimeout; }
            set { store.ScriptEngineTimeout = value; }
        }

        public void SetProperty(int propertyId, object propertyValue)
        {
            store.SetProperty(propertyId, propertyValue, null);
        }

        public void Submit(int flags)
        {
            store.Submit(flags, null);
        }

        public string TargetMachine
        {
            get { return store.TargetMachine; }
        }

        public void UpdateCache()
        {
            store.UpdateCache(null);
        }

        public bool Writable
        {
            get { return Convert.ToBoolean(store.Writable); }
        }
    }
}
